Attribute VB_Name = "basDPlay"
' DPlay, Copyright 1998, Patrice Scribe
' A simple DirectPlay3 sample
' Needs DirectX support v 0.6 or later, Win32 support v 0.2 or later
' Tested with serial connection only

Option Explicit

' Not declared properly in the Win32 type library ?
Private Declare Function lstrcpy Lib "kernel32" (a As Any, b As Any) As Long

Public dp As IDirectPlay3           ' Main DirectPlay object
Public ChoosenConnection As Long    ' Choosen connection

Dim CLSID_DirectPlay As Win32.GUID  ' DirectPlay class ID
Dim IID_IDirectPlay3A As Win32.GUID ' DirectPlay3A interface ID
Dim AppId As Win32.GUID             ' This app ID (created by uuidgen)

' Enumerated connections
Public Type TConnection
    Connection() As Byte    ' Connection data
    Name As String          ' Connection name
End Type
Public Connections(0 To 9) As TConnection
Public ConnectionCount As Long

' Enumerated sessions
Public Type TSession
    Description As DPSESSIONDESC2
    Name As String
    Password As String
End Type
Public Sessions(1 To 10) As TSession
Public SessionCount As Long

Sub Main()
    Dim ShortName As String     ' Player short name
    Dim LongName As String      ' Player long name
    Dim PlayerName As DPNAME    ' Player name structure
    Dim hEvent As Long          ' Event raised when incoming data
    Dim PlayerId As Long        ' Player id
    Dim Data As String          ' Data buffer for messages
    Dim DataSize As Long        ' Data length
    Dim MessageCount As Long    ' Number of messages
    ' Initialize GUID values
    IIDFromString StrPtr("{D1EB6D20-8923-11D0-9D97-00A0C90A43CB}"), CLSID_DirectPlay
    IIDFromString StrPtr("{133EFE41-32DC-11D0-9CFB-00A0C90A43CB}"), IID_IDirectPlay3A
    IIDFromString StrPtr("{17553DA0-02EE-11D2-A098-9C9501C1440E}"), AppId
    ' Create the DirectPlay object (DirectPlayCreate is said to be obsolete)
    CoCreateInstance CLSID_DirectPlay, 0, 1, IID_IDirectPlay3A, dp
    ' Enumerate connections
    dp.EnumConnections ByVal 0&, AddressOf EnumConnectionsCallback, ByVal 0&, 0
    ' Choose connection
    frmDPlay.Show vbModal
    If ChoosenConnection = -1 Then
        Set dp = Nothing
        End
    End If
    ' Initialize choosen connection
    dp.InitializeConnection Connections(ChoosenConnection).Connection(0), 0
    ' Enumerate sessions
    frmDPlayMsg.Show
    frmDPlayMsg.Message "Enumerating..."
    Dim sd As DPSESSIONDESC2
    sd.dwSize = Len(sd)
    sd.guidApplication = AppId
    dp.EnumSessions sd, 10000, AddressOf EnumSessionsCallback, ByVal 0&, DPENUMSESSIONS_AVAILABLE
    ' If no session found
    If SessionCount = 0 Then
        ' Create a session
        Dim SessionName As String
        SessionName = "VBA51"
        With sd
            .dwFlags = DPSESSION_KEEPALIVE
            .dwSize = Len(sd)
            .lpszSessionName = VarPtr(SessionName)
        End With
        dp.Open sd, DPOPEN_CREATE
        frmDPlayMsg.Message "Session created..."
    Else
        ' Join a session
        sd = Sessions(1).Description
        sd.lpszSessionName = VarPtr(Sessions(1).Name)
        dp.Open sd, DPOPEN_JOIN
        Debug.Print "Session joined..."
    End If
    ' Initialize player data
    ShortName = "Pat"
    LongName = "Patrice Scribe"
    PlayerName.dwSize = Len(PlayerName)
    PlayerName.lpszShortName = VarPtr(ShortName)
    PlayerName.lpszLongName = VarPtr(LongName)
    ' Create an event for incoming data
    hEvent = CreateEvent(ByVal 0&, 0, 0, vbNullString)
    ' Create player
    dp.CreatePlayer PlayerId, PlayerName, hEvent, ByVal 0, 0, 0
    frmDPlayMsg.Show
    While DoEvents() <> 0
        ' If event raised...
        If WaitForSingleObject(hEvent, 0) = 0 Then
            ' Read all messages
            dp.GetMessageCount PlayerId, MessageCount
            Do
                Data = Space$(100)
                DataSize = Len(Data)
                dp.Receive DPID_ALLPLAYERS, PlayerId, DPRECEIVE_ALL, ByVal Data, DataSize
                Data = Left$(Data, DataSize)
                MessageCount = MessageCount - 1
                frmDPlayMsg.Message "Received : " & Data
            Loop Until MessageCount = 0
        End If
        ' Send a random message when a key is pressed
        Data = ""
        If GetAsyncKeyState(vbKeySpace) <> 0 Then
            Select Case Int(Rnd * 4 + 1)
                Case 1
                    Data = "Left" & Chr$(0)
                Case 2
                    Data = "Up" & Chr$(0)
                Case 3
                    Data = "Down" & Chr$(0)
                Case 4
                    Data = "Right" & Chr$(0)
                Case Else
                    Debug.Assert False
            End Select
        End If
        If Data <> "" Then
            dp.Send PlayerId, DPID_ALLPLAYERS, 0, ByVal Data, Len(Data) * 2 ' Probably because VB strings are unicode
            frmDPlayMsg.Message "Sent : " & Data
        End If
    Wend
    ' Clean up is probably to improve
    dp.DestroyPlayer PlayerId
    dp.Close
    Set dp = Nothing
End Sub

Function EnumConnectionsCallback(ByRef lpGuid As Win32.GUID, ByVal lpConnection As Long, ByVal dwConnectionSize As Long, ByVal lpName As Long, ByVal dwFlags As Long, ByVal lpContext As Long) As Long
    Dim Name As DPNAME
    With Connections(ConnectionCount)
        ' Set connection data buffer size
        ReDim .Connection(dwConnectionSize)
        ' Copy connection data
        CopyMemory .Connection(0), ByVal lpConnection, dwConnectionSize
        ' Copy name structure
        CopyMemory Name, ByVal lpName, Len(Name)
        ' Set name size
        .Name = Space$(lstrlen(ByVal Name.lpszShortName))
        ' Copy name
        lstrcpy ByVal .Name, ByVal Name.lpszShortName
    End With
    ConnectionCount = ConnectionCount + 1
    EnumConnectionsCallback = 1    ' Continue enumeration
End Function

Public Function EnumSessionsCallback(ByVal lpSessionDescription As Long, TimeOut As Long, ByVal Flags As Long, ByVal lpContext As Long) As Long
    If Flags <> 0 Then Debug.Print "Timeout..."
    If lpSessionDescription <> 0 Then
        SessionCount = SessionCount + 1
        With Sessions(SessionCount)
            ' Copy connection data
            CopyMemory .Description, ByVal lpSessionDescription, Len(.Description)
            ' Set name size
            .Name = Space$(lstrlen(.Description.lpszSessionName))
            ' Copy name
            lstrcpy ByVal .Name, ByVal .Description.lpszSessionName
            If .Description.lpszPassword <> 0 Then
                ' Copy password
                .Password = Space$(lstrlen(ByVal .Description.lpszPassword))
                lstrcpy ByVal .Password, ByVal .Description.lpszPassword
            End If
        End With
    End If
    EnumSessionsCallback = 0    ' Stop enumeration
End Function
